##
# File: PdbxPrdExec.py
# Date: 1-Dec-2012
#
# Updates:
# 22-Jan-2013 jdw add option to export analog PRDCC definitions.
# 28-Jan-2013 jdw updated for change in return types for CvsSandboxAdmin() methods.
# 15-Oct-2013 jdw changed prdcc export to use prdcc_ in data_ section name.
##
"""
Execution environment for administrative operation on  BIRD PRD/FAMILY dictionary definitions.

"""
import sys
import os
import time
import traceback
import multiprocessing
from optparse import OptionParser

from mmcif_utils.bird.PdbxPrdIo import PdbxPrdIo
from mmcif_utils.bird.PdbxPrdCcIo import PdbxPrdCcIo
from mmcif_utils.bird.PdbxPrd import *
from mmcif_utils.bird.PdbxPrdUtils import PdbxPrdUtils
from wwpdb.utils.rcsb.CvsAdmin import CvsAdmin, CvsSandBoxAdmin


def readList(fn):
    try:
        idList = []
        ifh = open(fn, 'r')
        for line in ifh.readlines():
            tt = str(line[:-1]).strip()
            if len(tt) < 1 or tt.startswith("#"):
                continue
            idList.append(tt)
        return idList
    except Exception as e:
        return []


def exportList(dataList, fn=None):
    try:
        if fn is None:
            ofh = sys.stdout
            for datum in dataList:
                ofh.write("%s\n" % datum)
        else:
            ofh = open(fn, 'w')
            for datum in dataList:
                ofh.write("%s\n" % datum)
            ofh.close()
        #
        return True
    except Exception as e:
        return False


class PdbxPrdWorker(multiprocessing.Process):

    def __init__(self, taskQueue, resultQueue, pIndex=1, verbose=False, log=sys.stderr):
        multiprocessing.Process.__init__(self)
        self.__taskQueue = taskQueue
        self.__resultQueue = resultQueue

        self.__pIndex = pIndex
        #
        self.__verbose = verbose
        self.__lfh = log

    def run(self):
        processName = self.name
        while True:
            nextList = self.__taskQueue.get()
            if nextList is None:
                # end of queue condition
                if self.__verbose:
                    lt = time.strftime("%Y %m %d %H:%M:%S", time.localtime())
                    logger.info("+PdbLoaderExecWorker(run)  %s exiting at %s \n" % (processName, lt))
                break
            #

#            retList=pdbL.loadList(idList=nextList)
#            self.__resultQueue.put(retList)
        return


class PdbxPrdExec(object):
    """ Execution class for BIRD PRD/FAMILY dictionary definition administrative operations.

    """

    def __init__(self, topCachePath=None, topPrdCcCachePath=None, verbose=False, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = True
        self.__lfh = log
        self.__topCachePath = topCachePath
        self.__topPrdCcCachePath = topPrdCcCachePath
        self.__pathList = []
        self.__setup()
        #

    def __setup(self):
        self.__cvsRepositoryPath = os.getenv("CVS_REPOSITORY_PATH")
        self.__cvsRepositoryHost = os.getenv("CVS_REPOSITORY_HOST")
        self.__cvsUser = os.getenv("CVS_REPOSITORY_USER")
        self.__cvsPassword = os.getenv("CVS_REPOSITORY_PASSWORD")

    def runSequential(self):
        """Run load  operations sequentially
        """
        if self.__verbose:
            lt = time.strftime("%Y %m %d %H:%M:%S", time.localtime())
            logger.info("+PdbxPrdExec(runSequential) beginning tasks at %s\n" % lt)
        #

        if self.__verbose:
            lt = time.strftime("%Y %m %d %H:%M:%S", time.localtime())
            logger.info("+PdbxPrdExec(runSequential) task completed at %s\n" % lt)
        #

    def run(self, numProc=0):
        """Run all load operations in mulitprocessing mode.
        """
        #
        if self.__verbose:
            lt = time.strftime("%Y %m %d %H:%M:%S", time.localtime())
            logger.info("+PdbxPrdExec(run) beginning tasks at %s\n" % lt)

    def getIdsByStatus(self, selectList=['ALL']):
        """Read all repository files and return id codes corresponding to input status options
        """
        logger.info("\nStarting %s %s\n" % (self.__class__.__name__,
                                            sys._getframe().f_code.co_name))
        rL = []
        try:
            if selectList is None or len(selectList) < 1:
                return rL
            #
            prd = PdbxPrdIo(verbose=self.__verbose, log=self.__lfh)
            prd.setCachePath(self.__topCachePath)
            self.__pathList = prd.makeDefinitionPathList()
            for pth in self.__pathList:
                prd.setFilePath(pth)
            if (self.__verbose):
                logger.info("Repository %s read completed\n" % self.__topCachePath)
            #
            prdU = PdbxPrdUtils(prd, verbose=self.__verbose, log=self.__lfh)
            statusD = prdU.getStatus()
            if 'ALL' in selectList:
                rL = sorted(statusD.keys())
            else:
                for id in sorted(statusD.keys()):
                    if statusD[id] in selectList:
                        rL.append(id)
            return rL
        except Exception as e:
            if self.__verbose:
                logger.exception("Failing with %s" % str(e))
            return []

    def getStatus(self):
        pass

    def exportList(self, idList=None, outPath=None, stripFlag=False, suppressCompFile=None, releaseDate=None):
        """Export concatenated/split file of PRD definitions corresponding to input PRD id code list.

           Return True for success or False otherwise.
        """
        _mId = "(" + self.__class__.__name__ + "." + sys._getframe().f_code.co_name + ")"
        if self.__verbose:
            logger.info("\n%s Starting\n" % _mId)
        try:
            if idList is None or outPath is None:
                return False

            prd = PdbxPrdIo(verbose=self.__verbose, log=self.__lfh)
            prd.setCachePath(self.__topCachePath)
            for id in idList:
                prd.setPrdId(prdId=id)
            if self.__verbose:
                logger.info("%s Repository read completed\n" % _mId)
            #
            prdU = PdbxPrdUtils(prd, verbose=self.__verbose, log=self.__lfh)
            if suppressCompFile is not None:
                if self.__verbose:
                    logger.info("%s Suppressing chemical component id codes from file %s \n" % (_mId, suppressCompFile))
                ccList = readList(suppressCompFile)
                prdU.suppressComponentsFromList(idList=ccList)

            if releaseDate is not None:
                if self.__verbose:
                    logger.info("%s Assigning release release status and audit date %s\n" % (_mId, releaseDate))
                cD = prdU.setReleaseAndAudit(relDate=releaseDate)
            #
            prd.writeFile(outPath, applyConstraints=stripFlag)

        except Exception as e:
            if self.__verbose:
                logger.exception("Failing with %s" % str(e))
            return []

    def exportPrdCcList(self, idList=None, stripFlag=False, outPath=None):
        """Export concatenated/split file or PRDCC analog definitions corresponding to input id PRD ID code list.

           Return True for success or False otherwise.
        """
        _mId = "(" + self.__class__.__name__ + "." + sys._getframe().f_code.co_name + ")"
        if self.__verbose:
            logger.info("\n%s Starting\n" % _mId)
        try:
            if idList is None or outPath is None:
                return False
            prd = PdbxPrdCcIo(verbose=self.__verbose, log=self.__lfh)
            prd.setCachePath(self.__topPrdCcCachePath)
            for id in idList:
                # convert PRD_ to PRDCC_ id code
                myId = "PRDCC_" + id[4:]
                filePath = os.path.join(self.__topPrdCcCachePath, myId[-1], myId + '.cif')
                if os.access(filePath, os.R_OK):
                    prd.setFilePath(filePath=filePath, prdCcId=myId)
                else:
                    pass
            if self.__verbose:
                logger.info("%s Repository read completed\n" % _mId)
            #
            # Change the name of the data_ section to the PRDCC_ prefix.
            cList = prd.getCurrentContainerList()
            for c in cList:
                id = c.getName()
                newId = 'PRDCC_' + id[4:]
                c.setName(newId)
            #
            prd.writeFile(outPath, applyConstraints=stripFlag)

        except Exception as e:
            if self.__verbose:
                logger.exception("Failing with %s" % str(e))
            return []

    def getChemCompIdList(self, idList=None):
        """Get the list of referenced chemical component id codes in the input list of PRD defnitions.

           Return a list of chemical component 3-letter codes.
        """
        _mId = "(" + self.__class__.__name__ + "." + sys._getframe().f_code.co_name + ")"
        if self.__verbose:
            logger.info("\n%s Starting\n" % _mId)
        try:
            if idList is None:
                return False

            prd = PdbxPrdIo(verbose=self.__verbose, log=self.__lfh)
            prd.setCachePath(self.__topCachePath)
            for id in idList:
                prd.setPrdId(prdId=id)
            if self.__verbose:
                logger.info("%s Repository read completed\n" % _mId)
            #
            prdU = PdbxPrdUtils(prd, verbose=self.__verbose, log=self.__lfh)
            ccD = prdU.getReferencedComponents()
            #
            rD = {}
            for prdId, ccL in ccD.items():
                for cc in ccL:
                    rD[cc] = cc

            return sorted(rD.keys())

        except Exception as e:
            if self.__verbose:
                logger.exception("Failing with %s" % str(e))
            return []

    def releaseList(self, idList=None, releaseDate=None):
        """Set the release/audit records for selected definitions in the working copy of the repository.
           The working copy is first updated, the status/audit records changed, and changed definitions
           are committed to the repository.

           Return True for success or False otherwise.
        """
        _mId = "(" + self.__class__.__name__ + "." + sys._getframe().f_code.co_name + ")"
        if self.__verbose:
            logger.info("\n%s Starting using releaseDate %s\n" % (_mId, releaseDate))
        try:
            if idList is None or releaseDate is None:
                return False

            (wrkPath, projDir) = os.path.split(self.__topCachePath)

            vc = CvsSandBoxAdmin(tmpPath="./")
            vc.setRepositoryPath(host=self.__cvsRepositoryHost, path=self.__cvsRepositoryPath)
            vc.setAuthInfo(user=self.__cvsUser, password=self.__cvsPassword)
            vc.setSandBoxTopPath(wrkPath)

            # first update the working directory with the latest files from the repository
            #
            for id in idList:
                hash = id[-1]
                relFilePath = os.path.join(hash, id + '.cif')
                #
                vc.update(projDir, relFilePath)

                # update the status items --
                #
                prd = PdbxPrdIo(verbose=self.__verbose, log=self.__lfh)
                prd.setCachePath(self.__topCachePath)
                prd.setPrdId(prdId=id)
                #
                prdU = PdbxPrdUtils(prd, verbose=self.__verbose, log=self.__lfh)
                if releaseDate is not None:
                    if self.__verbose:
                        logger.info("%s  %s assigning release release status and audit date %s\n" % (_mId, id, releaseDate))
                    cD = prdU.setReleaseAndAudit(relDate=releaseDate)
                #
                outPath = os.path.join(wrkPath, projDir, relFilePath)
                prd.writeFile(outPath, applyConstraints=False)
                #
                vc.commit(projDir, relFilePath)
            #
            if (not self.__debug):
                vc.cleanup()
            return True
        except Exception as e:
            if self.__verbose:
                logger.info("%s  error in release list processing\n" % _mId)
                logger.exception("Failing with %s" % str(e))
            return False

    def checkOut(self):
        """Checkout a working copy of the project within the repository.

           Return True for success or False otherwise.
        """
        _mId = "(" + self.__class__.__name__ + "." + sys._getframe().f_code.co_name + ")"
        if self.__verbose:
            logger.info("\n%s Starting\n" % _mId)
        try:
            (wrkPath, projDir) = os.path.split(self.__topCachePath)

            vc = CvsSandBoxAdmin(tmpPath="./", verbose=self.__verbose, log=self.__lfh)
            vc.setRepositoryPath(host=self.__cvsRepositoryHost, path=self.__cvsRepositoryPath)
            vc.setAuthInfo(user=self.__cvsUser, password=self.__cvsPassword)
            vc.setSandBoxTopPath(wrkPath)

            ok, text = vc.checkOut(projectPath=projDir)
            if self.__verbose:
                logger.info("\n%s CVS checkout output:\n%s\n" % (_mId, text))

            vc.cleanup()
            return True
        except Exception as e:
            if self.__verbose:
                logger.exception("Failing with %s" % str(e))
            return False


def main():
    _mId = "(" + sys._getframe().f_code.co_name + ")"
    usage = "usage: %prog [options]"
    #
    parser = OptionParser(usage)
    # parser.add_option("--numproc", default=10, type="int", dest="numProc", help="Number of concurent processes to run")

    parser.add_option("-v", "--verbose", default=False,
                      action="store_true", dest="verbose", help="Enable verbose output")

    parser.add_option("--prd_id", default=None, type="string", dest="prdId", help="Target PRD ID Code")

    parser.add_option("--set_cvs_release_status", dest="setReleaseStatusCvsOp", action='store_true', default=False,
                      help="Set release status and audit records in cvs repository")

    parser.add_option("--cvs_checkout", dest="cvsCheckOutOp", action='store_true', default=False,
                      help="Checkout a working copy of the current repository and project")

    parser.add_option("--strip_internal_items", dest="stripInternalOp", action='store_true', default=False,
                      help="Exclude internal data items in exported files")

    parser.add_option("--export_release_date", dest="exportReleaseDate", default=None, type="string",
                      help="Assign release status and audit records with YYYY-MM-DD in exported definitions")

    parser.add_option("--export_public_targzfile", dest="exportPubTgzOp", action='store_true', default=False,
                      help="Export split gzipped tarfile of PRDpublic definitions")

    parser.add_option("--export_public_file", dest="exportPubOp", action='store_true', default=False,
                      help="Export concatenated file of PRD public definitions")

    parser.add_option("--export_public_prdcc_targzfile", dest="exportPubPrdCcTgzOp", action='store_true', default=False,
                      help="Export split gzipped tarfile of PRDCC public definitions")

    parser.add_option("--export_public_prdcc_file", dest="exportPubPrdCcOp", action='store_true', default=False,
                      help="Export concatenated file of PRDCC public definitions")

    parser.add_option("--export_from_list", dest="exportListOp", action='store_true', default=False,
                      help="Export concatenated/split file from input id list")

    parser.add_option("--list_id_codes", dest="listIdCodesOp", default=None, type="string", help="List Id codes by status (ALL,REL,HOLD,..)")

    parser.add_option("--list_chem_comp_id_codes", dest="listChemCompIdCodesOp", action='store_true', default=False,
                      help="List id codes for referenced chemical components")

    parser.add_option("--suppress_chem_comp_file", dest="suppressCompFile", default=None, type="string",
                      help="File containing chemical component identifiers to be suppressed")

    parser.add_option("-o", "--outfile", dest="outputFilePath", default=None, type="string", help="Output file path")

    parser.add_option("-i", "--inpfile", dest="inputFilePath", default=None, type="string", help="Input file path")

    parser.add_option("--cvs_path", dest="topCachePath", default="/data/components/prd-v3", type="string", help="PRD CVS working directory path")

    parser.add_option("--cvs_prdcc_path", dest="topPrdCcCachePath", default="/data/components/prdcc-v3", type="string", help="PRDCC CVS working directory path")

    (options, args) = parser.parse_args()

    if (options.verbose):
        lt = time.strftime("%Y %m %d %H:%M:%S", time.localtime())
        sys.stdout.write("%s begin tasks at %s\n" % (_mId, lt))

    if options.cvsCheckOutOp:
        pX = PdbxPrdExec(topCachePath=options.topCachePath, verbose=options.verbose, log=sys.stdout)
        ok = pX.checkOut()

    elif options.setReleaseStatusCvsOp and (options.prdId is not None or options.inputFilePath is not None):

        if options.prdId is not None:
            if options.verbose:
                sys.stdout.write("%s set release status and audit records for %s\n" % (_mId, options.prdId))
            idList = [options.prdId]
        elif options.inputFilePath is not None:
            if options.verbose:
                sys.stdout.write("%s set release status and audit records for id code list file  %s\n" % (_mId, options.inputFilePath))
            idList = readList(inputFilePath)
        else:
            pass
        #
        if options.exportReleaseDate is not None:
            dt = options.exportReleaseDate
        else:
            dt = time.strftime("%Y-%m-%d", time.localtime())
        pX = PdbxPrdExec(topCachePath=options.topCachePath, verbose=options.verbose, log=sys.stdout)
        ok = pX.releaseList(idList=idList, releaseDate=dt)

    elif (options.exportPubOp or options.exportPubTgzOp) and options.outputFilePath is not None:
        if options.exportPubTgzOp and options.outputFilePath.endswith(".tar.gz"):
            sys.stdout.write("%s export split gzipped tarfile of public definitions in %s\n" % (_mId, options.outputFilePath))
        elif options.exportPubOp and options.outputFilePath is not None:
            sys.stdout.write("%s export concatenated file of public definitions in %s\n" % (_mId, options.outputFilePath))
        else:
            sys.stdout.write("%s bad input for export %s\n" % (_mId, options.outputFilePath))
            exit(2)

        pX = PdbxPrdExec(topCachePath=options.topCachePath, verbose=options.verbose, log=sys.stdout)
        idList = pX.getIdsByStatus(selectList=['REL', 'OBS'])

        ok = pX.exportList(idList=idList,
                           outPath=options.outputFilePath,
                           stripFlag=True,
                           suppressCompFile=options.suppressCompFile,
                           releaseDate=options.exportReleaseDate)

    elif (options.exportPubPrdCcOp or options.exportPubPrdCcTgzOp) and options.outputFilePath is not None:
        if options.exportPubPrdCcTgzOp and options.outputFilePath.endswith(".tar.gz"):
            sys.stdout.write("%s export split gzipped tarfile of public PRDCC analog definitions in %s\n" % (_mId, options.outputFilePath))
        elif options.exportPubPrdCcOp and options.outputFilePath is not None:
            sys.stdout.write("%s export concatenated file of public PRDCC analog definitions in %s\n" % (_mId, options.outputFilePath))
        else:
            sys.stdout.write("%s bad input for export %s\n" % (_mId, options.outputFilePath))
            exit(2)

        pX = PdbxPrdExec(topCachePath=options.topCachePath, topPrdCcCachePath=options.topPrdCcCachePath, verbose=options.verbose, log=sys.stdout)
        idList = pX.getIdsByStatus(selectList=['REL', 'OBS'])

        ok = pX.exportPrdCcList(idList=idList, outPath=options.outputFilePath, stripFlag=False)

    elif options.exportListOp and options.inputFilePath is not None:
        if options.verbose:
            sys.stdout.write("%s export concatenated file of definitions from input id file %s\n" % (_mId, options.inputFilePath))

        idList = readList(options.inputFilePath)

        pX = PdbxPrdExec(topCachePath=options.topCachePath, verbose=options.verbose, log=sys.stdout)
        ok = pX.exportList(idList=idList,
                           outPath=options.outputFilePath,
                           stripFlag=options.stripInternalOp,
                           suppressCompFile=options.suppressCompFile,
                           releaseDate=options.exportReleaseDate)

    elif options.listIdCodesOp is not None:
        if options.verbose:
            sys.stdout.write("%s list codes with status %s\n" % (_mId, options.listIdCodesOp))

        pX = PdbxPrdExec(topCachePath=options.topCachePath, verbose=options.verbose, log=sys.stdout)
        sL = pX.getIdsByStatus(selectList=[options.listIdCodesOp])

        if options.outputFilePath is not None:
            exportList(sL, fn=options.outputFilePath)
        else:
            exportList(sL)

    elif options.listChemCompIdCodesOp is not None:
        if options.verbose:
            sys.stdout.write("%s list id codes for referenced chemical components\n" % (_mId))

        pX = PdbxPrdExec(topCachePath=options.topCachePath, verbose=options.verbose, log=sys.stdout)
        prdIdList = pX.getIdsByStatus(selectList=['REL', 'OBS'])
        ccL = pX.getChemCompIdList(idList=prdIdList)
        if options.outputFilePath is not None:
            exportList(ccL, fn=options.outputFilePath)
        else:
            exportList(ccL)
    else:
        pass

    if (options.verbose):
        lt = time.strftime("%Y %m %d %H:%M:%S", time.localtime())
        sys.stdout.write("%s completed tasks at %s\n" % (_mId, lt))


if __name__ == "__main__":
    main()



##
# File: DictionaryMapperIo.py
# Date: 11-Jun-2015  John Westbrook
#
# Update:
##
"""
Manage IO operations for category/item-level mapping data files.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"

import sys


from mmcif_utils.style.PdbxStyleIoUtil import PdbxStyleIoUtil
from mmcif_utils.style.PdbxItemMappingCategoryStyle import PdbxItemMappingCategoryStyle
from mmcif.io.IoAdapterPy import IoAdapterPy as IoAdapter


class DictionaryMapperIo(PdbxStyleIoUtil):

    ''' Methods for accessing category and item-level mapping information --- .

    '''

    def __init__(self, verbose=True, log=sys.stderr):
        super(DictionaryMapperIo, self).__init__(styleObject=PdbxItemMappingCategoryStyle(), IoAdapter=IoAdapter(), verbose=verbose, log=log)

        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        self.__mappingFilePath = None

    def getCategory(self, catName='pdbx_dict_item_mapping'):
        return self.getAttribDictList(catName)

    def setMapFilePath(self, filePath):
        """ Specify the mapping file path.
        """
        self.__mappingFilePath = filePath
        if self.readFile(self.__mappingFilePath):
            return self.setContainer(containerIndex=0)
        else:
            return False

    def complyStyle(self):
        return self.testStyleComplete(self.__lfh)

    def setBlock(self, blockId):
        return self.setContainer(containerName=blockId)

    def newBlock(self, blockId):
        return self.newContainer(containerName=blockId)

    def update(self, catName, attributeName, value, iRow=0):
        return self.updateAttribute(catName, attributeName, value, iRow=iRow)

    def write(self, filePath):
        return self.writeFile(filePath)





##
# File:    PdbxMessage.py
# Author:  J. Westbrook
# Date:    6-Aug-2013
# Version: 0.001 Initial version
#
# Updates:
#     7-Aug-2013 jdw  added file reference data
#    15-Aug-2013 jdw  added class PdbxMessageStatus()
#    15-Aug-2013 jdw  chain read status to send status in PdbxMessageInfo()
#    15-Aug-2013 rps  correcting typo for 'send_status' default value
#    12-Sep-2013 rps  switching from use of localtime to GMT time for standardization of message timestamps across sites.
#    30-Oct-2013 rps  support for UI feature for classifying messages (e.g. "action required" or "unread")
#    29-Jul-2014 rps  added set/getUploadFileName to support multiple file references for a single message
#                        (i.e. so that original name can be used to more easily distinguish one file reference from another).
#    09-Sep-2014 rps  Added PdbxMessageOrigCommReference.
##
"""
A class for managing message data --

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.001"

import sys
import time

import logging
logger = logging.getLogger(__name__)

from mmcif_utils.style.PdbxMessageCategoryStyle import PdbxMessageCategoryStyle


class PdbxMessageInfo(object):

    def __init__(self, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__lfh = log
        self.__stObj = PdbxMessageCategoryStyle()
        self.__attributeIdList = self.__stObj.getAttributeNameList('pdbx_deposition_message_info')
        self.__D = self.__setup()

    def __getMessageId(self):
        try:
            import uuid
            return str(uuid.uuid4())
        except Exception as e:
            import random
            import string
            chars = string.ascii_lowercase + string.digits
            string = ''.join(random.choice(chars) for x in range(8))
            string += '-' + ''.join(random.choice(chars) for x in range(4))
            string += '-' + ''.join(random.choice(chars) for x in range(4))
            string += '-' + ''.join(random.choice(chars) for x in range(4))
            string += '-' + ''.join(random.choice(chars) for x in range(12))
            return string

    def __setup(self):
        # initialize the dictionary
        d = {}
        for ky in self.__attributeIdList:
            d[ky] = None
        #
        # set default values -
        gmtTmStmp = time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime())
        d["message_id"] = self.__getMessageId()
        d["parent_message_id"] = d["message_id"]
        d["timestamp"] = gmtTmStmp
        d["message_type"] = "text"
        d["send_status"] = "N"
        #
        return d

    def get(self):
        return self.__D

    def set(self, messageDict={}):
        d = {}
        d = self.__setup()
        for k, v in messageDict.items():
            # only add know attributes --
            if k in self.__attributeIdList:
                d[k] = v
        #
        self.__D = d
        return d

    def write(self, ofh=sys.stdout):
        for attributeId in self.__attributeIdList:
            ofh.write("%-30s : %-s\n" % (attributeId, self.__D[attributeId]))

    def setOrdinalId(self, id=None):
        self.__D['ordinal_id'] = id

    def getOrdinalId(self):
        return self.__D["ordinal_id"]

    def setMessageId(self, id=None):
        if id is None:
            d["message_id"] = str(uidd.uidd4())

    def getMessageId(self):
        return self.__D["message_id"]

    def setDepositionId(self, id=None):
        self.__D['deposition_data_set_id'] = id

    def getDepositionId(self):
        return self.__D['deposition_data_set_id']

    def setSender(self, sender=None):
        self.__D["sender"] = sender

    def getSender(self):
        return self.__D["sender"]

    def setContextType(self, type=None):
        self.__D["context_type"] = type

    def getContextType(self):
        return self.__D["context_type"]

    def setContextValue(self, val=None):
        self.__D["context_value"] = val

    def getContextValue(self):
        return self.__D["context_value"]

    def setParentMessageId(self, id=None):
        self.__D["parent_message_id"] = id

    def getParentMessageId(self):
        return self.__D["parent_message_id"]

    def setMessageType(self, type=None):
        self.__D["message_type"] = type

    def getMessageType(self):
        return self.__D["message_type"]

    def setMessageSubject(self, txt=None):
        self.__D["message_subject"] = txt

    def getMessageSubject(self):
        return self.__D["message_subject"]

    def setMessageText(self, txt=None):
        self.__D["message_text"] = txt

    def getMessageText(self):
        return self.__D["message_text"]

    def setSendStatus(self, status='N'):
        self.__D["send_status"] = status

    def getSendStatus(self):
        return self.__D["send_status"]

    def getValueList(self, attributeIdList=[]):
        vL = []
        for attributeId in attributeIdList:
            if attributeId in self.__D:
                vL.append(self.__D[attributeId])
            else:
                vL.append(None)
        return vL


class PdbxMessageFileReference(object):

    def __init__(self, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__lfh = log
        self.__stObj = PdbxMessageCategoryStyle()
        self.__attributeIdList = self.__stObj.getAttributeNameList('pdbx_deposition_message_file_reference')
        self.__D = self.__setup()

    def __setup(self):
        # initialize the dictionary
        d = {}
        for ky in self.__attributeIdList:
            d[ky] = None
        #
        #
        return d

    def get(self):
        return self.__D

    def set(self, messageDict={}):
        d = {}
        d = self.__setup()
        for k, v in messageDict.items():
            # only add know attributes --
            if k in self.__attributeIdList:
                d[k] = v
        #
        self.__D = d
        return d

    def write(self, ofh=sys.stdout):
        for attributeId in self.__attributeIdList:
            ofh.write("%-30s : %-s\n" % (attributeId, self.__D[attributeId]))

    def setOrdinalId(self, id=None):
        self.__D['ordinal_id'] = id

    def getOrdinalId(self):
        return self.__D["ordinal_id"]

    def setMessageId(self, id=None):
        self.__D["message_id"] = id

    def getMessageId(self):
        return self.__D["message_id"]

    def setDepositionId(self, id=None):
        self.__D['deposition_data_set_id'] = id

    def getDepositionId(self):
        return self.__D['deposition_data_set_id']

    def setContentType(self, type=None):
        self.__D["content_type"] = type

    def getContentType(self):
        return self.__D["content_type"]

    def setContentFormat(self, type=None):
        self.__D["content_format"] = type

    def getContentFormat(self):
        return self.__D["content_format"]

    def setUploadFileName(self, fileName=""):
        self.__D["upload_file_name"] = fileName

    def getUploadFileName(self):
        return self.__D["upload_file_name"]

    def setStorageType(self, type=None):
        self.__D["storage_type"] = type

    def getStorageType(self):
        return self.__D["storage_type"]

    def setPartitionNumber(self, val=None):
        self.__D["partition_number"] = val

    def getPartitionNumber(self):
        return self.__D["partition_number"]

    def setVersionId(self, val=None):
        self.__D["version_id"] = val

    def getVersionId(self):
        return self.__D["version_id"]


class PdbxMessageOrigCommReference(object):

    def __init__(self, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__lfh = log
        self.__stObj = PdbxMessageCategoryStyle()
        self.__attributeIdList = self.__stObj.getAttributeNameList('pdbx_deposition_message_origcomm_reference')
        self.__D = self.__setup()

    def __setup(self):
        # initialize the dictionary
        d = {}
        for ky in self.__attributeIdList:
            d[ky] = None
        #
        #
        return d

    def get(self):
        return self.__D

    def set(self, messageDict={}):
        d = {}
        d = self.__setup()
        for k, v in messageDict.items():
            # only add know attributes --
            if k in self.__attributeIdList:
                d[k] = v
        #
        self.__D = d
        return d

    def write(self, ofh=sys.stdout):
        for attributeId in self.__attributeIdList:
            ofh.write("%-30s : %-s\n" % (attributeId, self.__D[attributeId]))

    def setOrdinalId(self, id=None):
        self.__D['ordinal_id'] = id

    def getOrdinalId(self):
        return self.__D["ordinal_id"]

    def setMessageId(self, id=None):
        self.__D["message_id"] = id

    def getMessageId(self):
        return self.__D["message_id"]

    def setDepositionId(self, id=None):
        self.__D['deposition_data_set_id'] = id

    def getDepositionId(self):
        return self.__D['deposition_data_set_id']

    def setOrigMessageId(self, origId=None):
        self.__D["orig_message_id"] = origId

    def getOrigMessageId(self):
        return self.__D["orig_message_id"]

    def setOrigDepositionId(self, origDepId=None):
        self.__D["orig_deposition_data_set_id"] = origDepId

    def getOrigDepositionId(self):
        return self.__D["orig_deposition_data_set_id"]

    def setOrigTimeStamp(self, origTimeStamp=None):
        self.__D["orig_timestamp"] = origTimeStamp

    def getOrigTimeStamp(self):
        return self.__D["orig_timestamp"]

    def setOrigSender(self, origSender=None):
        self.__D["orig_sender"] = origSender

    def getOrigSender(self):
        return self.__D["orig_sender"]

    def setOrigRecipient(self, origRecipient=None):
        self.__D["orig_recipient"] = origRecipient

    def getOrigRecipient(self):
        return self.__D["orig_recipient"]

    def setOrigMessageSubject(self, subject=None):
        self.__D["orig_message_subject"] = subject

    def getOrigMessageSubject(self):
        return self.__D["orig_message_subject"]

    def setOrigAttachments(self, origAttachments=None):
        self.__D["orig_attachments"] = origAttachments

    def getOrigAttachments(self):
        return self.__D["orig_attachments"]


class PdbxMessageStatus(object):

    def __init__(self, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__lfh = log
        self.__stObj = PdbxMessageCategoryStyle()
        self.__attributeIdList = self.__stObj.getAttributeNameList('pdbx_deposition_message_status')
        self.__D = self.__setup()

    def __setup(self):
        # initialize the dictionary
        d = {}
        for ky in self.__attributeIdList:
            d[ky] = None
        #
        #
        return d

    def get(self):
        return self.__D

    def set(self, messageDict={}):
        d = {}
        d = self.__setup()
        for k, v in messageDict.items():
            # only add know attributes --
            if k in self.__attributeIdList:
                d[k] = v
        #
        self.__D = d
        return d

    def write(self, ofh=sys.stdout):
        for attributeId in self.__attributeIdList:
            ofh.write("%-30s : %-s\n" % (attributeId, self.__D[attributeId]))

    def setMessageId(self, id=None):
        self.__D["message_id"] = id

    def getMessageId(self):
        return self.__D["message_id"]

    def setDepositionId(self, id=None):
        self.__D['deposition_data_set_id'] = id

    def getDepositionId(self):
        return self.__D['deposition_data_set_id']

    def setReadStatus(self, status='N'):
        self.__D["read_status"] = status

    def getReadStatus(self):
        return self.__D["read_status"]

    def setActionReqdStatus(self, status='N'):
        self.__D["action_reqd"] = status

    def getActionReqdStatus(self):
        return self.__D["action_reqd"]

    def setReadyForRelStatus(self, status='N'):
        self.__D["for_release"] = status

    def getReadyForRelStatus(self):
        return self.__D["for_release"]



##
# File: PdbxConnect.py
# Date: 05-June-2010
#
#
# Updated:
#  23-Oct-2012 jdw  Update path and reorganize
##
"""

A collection of classes to determine connectivity based on atom serial numbers.


"""

__docformat__ = "restructuredtext en"
__author__    = "John Westbrook"
__email__     = "john.westbrook@rcsb.org"
__license__   = "Apache 2.0"
__version__   = "V0.01"

import sys,time,os,traceback
from operator import itemgetter
from mmcif.api.PdbxContainers import *
from mmcif.api.DataCategory   import DataCategory
from mmcif_utils.cc.PdbxChemComp       import PdbxChemCompReader

def numeric_compare(x,y):
    return int(x)-int(y)

class PdbxConnect(object):
    """
    """
    def __init__(self,topCachePath='/data/components/ligand-dict-v3', verbose=True,log=sys.stdout):
        self.__verbose=verbose
        self.__debug=False
        self.__lfh=log
        self.__topCachePath=topCachePath
        self.__dBlock=None
        self.__sC=':'
        self.__atomSiteMap = {'ID'          : '_atom_site.id',
                              'ATOM_NAME'   : '_atom_site.auth_atom_id',
                              'ALT_LOC'     : '_atom_site.label_alt_id',
                              'COMP_ID'     : '_atom_site.auth_comp_id',
                              'ASYM_ID'     : '_atom_site.auth_asym_id',
                              'SEQ_NUM'     : '_atom_site.auth_seq_id',
                              'INS_CODE'    : '_atom_site.pdbx_PDB_ins_code',
                              'MODEL_NUM'   : '_atom_site.pdbx_PDB_model_num'
                              }
        self.__stdCcList = ["ALA","ARG","ASN","ASP","ASX","CYS","GLN","GLU", "GLX",
                            "GLY","HIS","ILE","LEU","LYS","MET","PHE","PRO", "SER",
                            "THR","TRP","TYR","VAL", "DA", "DC", "DG", "DT",  "DU",
                            "DI",   "A",  "C",  "G",  "I",  "T",  "U", "HOH","UNK", "UNL"]


        self.__structConnMap = {
            'ID_CONN'     : '_struct_conn.id',
            'CONN_TYPE'   : '_struct_conn.conn_type_id',
            'ATOM_NAME_1' : '_struct_conn.ptnr1_label_atom_id',
            'ALT_LOC_1'   : '_struct_conn.pdbx_ptnr1_label_alt_id',
            'COMP_ID_1'   : '_struct_conn.ptnr1_auth_comp_id',
            'ASYM_ID_1'   : '_struct_conn.ptnr1_auth_asym_id',
            'SEQ_NUM_1'   : '_struct_conn.ptnr1_auth_seq_id',
            'INS_CODE_1'  : '_struct_conn.pdbx_ptnr1_PDB_ins_code',
            'SYM_OP_1'    : '_struct_conn.ptnr1_symmetry',
            'ATOM_NAME_2' : '_struct_conn.ptnr2_label_atom_id',
            'ALT_LOC_2'   : '_struct_conn.pdbx_ptnr2_label_alt_id',
            'COMP_ID_2'   : '_struct_conn.ptnr2_auth_comp_id',
            'ASYM_ID_2'   : '_struct_conn.ptnr2_auth_asym_id',
            'SEQ_NUM_2'   : '_struct_conn.ptnr2_auth_seq_id',
            'INS_CODE_2'  : '_struct_conn.pdbx_ptnr2_PDB_ins_code',
            'SYM_OP_2'    : '_struct_conn.ptnr2_symmetry',
            'MODEL_NUM'   : '_struct_conn.pdbx_PDB_model_num',
            'VALUE_DIST' : '_struct_conn.pdbx_dist_value'
            }
        """
        Example intermolecular contact.
loop_
_struct_conn.id
_struct_conn.conn_type_id
_struct_conn.pdbx_PDB_id
_struct_conn.ptnr1_label_asym_id
_struct_conn.ptnr1_label_comp_id
_struct_conn.ptnr1_label_seq_id
_struct_conn.ptnr1_label_atom_id
_struct_conn.pdbx_ptnr1_label_alt_id
_struct_conn.pdbx_ptnr1_PDB_ins_code
_struct_conn.pdbx_ptnr1_standard_comp_id
_struct_conn.ptnr1_symmetry
_struct_conn.ptnr2_label_asym_id
_struct_conn.ptnr2_label_comp_id
_struct_conn.ptnr2_label_seq_id
_struct_conn.ptnr2_label_atom_id
_struct_conn.pdbx_ptnr2_label_alt_id
_struct_conn.pdbx_ptnr2_PDB_ins_code
_struct_conn.ptnr1_auth_asym_id
_struct_conn.ptnr1_auth_comp_id
_struct_conn.ptnr1_auth_seq_id
_struct_conn.ptnr2_auth_asym_id
_struct_conn.ptnr2_auth_comp_id
_struct_conn.ptnr2_auth_seq_id
_struct_conn.ptnr2_symmetry
_struct_conn.pdbx_ptnr3_label_atom_id
_struct_conn.pdbx_ptnr3_label_seq_id
_struct_conn.pdbx_ptnr3_label_comp_id
_struct_conn.pdbx_ptnr3_label_asym_id
_struct_conn.pdbx_ptnr3_label_alt_id
_struct_conn.pdbx_ptnr3_PDB_ins_code
_struct_conn.details
_struct_conn.pdbx_dist_value
disulf1 disulf ? A CYS 23 SG ? ? ? 1_555 A CYS 88  SG ? ? A CYS 23 A CYS 88  1_555 ? ? ? ? ? ? ? 2.011
disulf2 disulf ? B CYS 22 SG ? ? ? 1_555 B CYS 95  SG ? ? B CYS 22 B CYS 95  1_555 ? ? ? ? ? ? ? 2.012
disulf3 disulf ? C CYS 6  SG ? ? ? 1_555 C CYS 127 SG ? ? C CYS 6  C CYS 127 1_555 ? ? ? ? ? ? ? 2.046
disulf4 disulf ? C CYS 30 SG ? ? ? 1_555 C CYS 115 SG ? ? C CYS 30 C CYS 115 1_555 ? ? ? ? ? ? ? 2.022
disulf5 disulf ? C CYS 64 SG ? ? ? 1_555 C CYS 80  SG ? ? C CYS 64 C CYS 80  1_555 ? ? ? ? ? ? ? 2.026
disulf6 disulf ? C CYS 76 SG ? ? ? 1_555 C CYS 94  SG ? ? C CYS 76 C CYS 94  1_555 ? ? ? ? ? ? ? 2.023

        """

        self.__clear();

    def __clear(self):
        self.__uniqCcList=[]
        self.__nsCcDict={}
        self.__nsCcList=[]
        self.__nsCcAltDict={}
        self.__bondTemplates={}
        self.__linkCcList=[]
        self.__linkCcSig=[]
        self.__linkCcDict={}
        self.__modelList=[]

    def setAtomSiteBlock(self,block=None):
        self.__clear()
        return self.__setDataBlock(block)

    def __setDataBlock(self,dataBlock=None):
        """
        """
        ok=False
        try:
            if (dataBlock.getType() == 'data'):
                self.__dBlock=dataBlock
                ok=True
            else:
                self.__dBlock=None
        except Exception as e:
            logger.exception("Failing with %s" % str(e))
            pass

        return ok

    def getNonStandardData(self,catName='atom_site'):
        """Return a list of bonds and types.
        """

        catObj=self.__dBlock.getObj(catName)
        nRows=catObj.getRowCount()

        itDict={}
        itNameList=catObj.getItemNameList()
        for idxIt,itName in enumerate(itNameList):
            itDict[itName]=idxIt
        #
        cM={}
        for k,v in self.__atomSiteMap.items():
            if itDict.has_key(v):
                cM[k]=itDict[v]
            else:
                cM[k]=-1
        #
        self.__clear()
        #
        rowList=catObj.getRowList()
        for row in rowList:
            if (row[cM['COMP_ID']] not in self.__stdCcList):
                rSig=self.__getCompSignatureAtomSite(row,cM)
                if not self.__nsCcDict.has_key(rSig):
                    self.__nsCcDict[rSig]={}
                    self.__nsCcList.append(rSig)
                    self.__nsCcAltDict[rSig]=[]
                    if row[cM['COMP_ID']] not in self.__uniqCcList:
                        self.__uniqCcList.append(row[cM['COMP_ID']])

                aSig=row[cM['ATOM_NAME']] + self.__sC + row[cM['ALT_LOC']]
                self.__nsCcDict[rSig][aSig]=row[cM['ID']]
                if row[cM['ALT_LOC']] not in self.__nsCcAltDict[rSig]:
                    self.__nsCcAltDict[rSig].append(row[cM['ALT_LOC']])

        if (self.__debug):
            logger.info("\n+DEBUG - __nsCcList %r\n" % self.__nsCcList)
            logger.info("\n+DEBUG - __nsCcDict %r\n" % self.__nsCcDict.items())


        self.__getBondTemplates()


    def getNonStandardDataAndLinks(self,catName='atom_site'):
        """Return a list of bonds and types.
        """

        catObj=self.__dBlock.getObj(catName)
        nRows=catObj.getRowCount()

        itDict={}
        itNameList=catObj.getItemNameList()
        for idxIt,itName in enumerate(itNameList):
            itDict[itName]=idxIt
        #
        cM={}
        for k,v in self.__atomSiteMap.items():
            if itDict.has_key(v):
                cM[k]=itDict[v]
            else:
                cM[k]=-1
        #
        self.__clear()
        #
        rowList=catObj.getRowList()
        for row in rowList:
            rSigNM=self.__getCompSignatureNoModel(row,cM)
            if (rSigNM in self.__linkCcSigList):
                rSig=self.__getCompSignatureAtomSite(row,cM)
                if not self.__linkCcDict.has_key(rSig):
                    self.__linkCcDict[rSig]={}
                aSig=row[cM['ATOM_NAME']] + self.__sC + row[cM['ALT_LOC']]
                self.__linkCcDict[rSig][aSig]=row[cM['ID']]

            if (row[cM['COMP_ID']] not in self.__stdCcList):
                rSig=self.__getCompSignatureAtomSite(row,cM)
                if (self.__debug):
                    logger.info("\n+DEBUG -rSig %r model %r\n" % (rSig,row[cM['MODEL_NUM']]))
                if not self.__nsCcDict.has_key(rSig):
                    self.__nsCcDict[rSig]={}
                    self.__nsCcList.append(rSig)
                    self.__nsCcAltDict[rSig]=[]
                    if row[cM['COMP_ID']] not in self.__uniqCcList:
                        self.__uniqCcList.append(row[cM['COMP_ID']])

                aSig=row[cM['ATOM_NAME']] + self.__sC + row[cM['ALT_LOC']]
                self.__nsCcDict[rSig][aSig]=row[cM['ID']]

                if row[cM['ALT_LOC']] not in self.__nsCcAltDict[rSig]:
                    self.__nsCcAltDict[rSig].append(row[cM['ALT_LOC']])

            modNum=self.__getModelNum(row[cM['MODEL_NUM']])
            if (modNum not in self.__modelList):
                self.__modelList.append(modNum)



        if (self.__debug):
            logger.info("\n+DEBUG - __nsCcList %r\n" % self.__nsCcList)
            logger.info("\n+DEBUG - __nsCcDict %r\n" % self.__nsCcDict.items())


        self.__getBondTemplates()


    def __getBondTemplates(self):
        for ccId in self.__uniqCcList:
            bL=[]
            try:
                cc=PdbxChemCompReader(verbose=self.__verbose,log=self.__lfh)
                cc.setCachePath(self.__topCachePath)
                cc.setCompId(compId=ccId)
                cc.getComp()
                bL=cc.getBondList()
            except Exception as e:
                logger.info("+ERROR - No component definition for %s\n" % ccId)
                logger.exception("Failing with %s" % str(e))
                continue

            if len(bL) > 0:
                self.__bondTemplates[ccId]=bL

    def __getModelNum(self,tString):
        """ Regularize the model number to 1 it is missing or null.
        """
        try:
            modNum= int(str(tString))
        except Exception as e:
            modNum=1

        return str(modNum)

    def __getCompSignatureAtomSite(self,row,cM):
        """ Create a unique signature for this component -
        """
        modNum=self.__getModelNum(row[cM['MODEL_NUM']])

        rSig = row[cM['COMP_ID']] + self.__sC + row[cM['ASYM_ID']] + self.__sC + row[cM['SEQ_NUM']] + self.__sC + row[cM['INS_CODE']] + self.__sC + str(modNum)
        return rSig

    def __getCompSignatureNoModel(self,row,cM):
        """ Create a unique signature for this component -
        """
        rSig = row[cM['COMP_ID']] + self.__sC + row[cM['ASYM_ID']] + self.__sC + row[cM['SEQ_NUM']] + self.__sC + row[cM['INS_CODE']]
        return rSig


    def __getCompSignatureStructConn(self,row,cM):
        """ Create a unique signatures for the the components in this linkage.
        """

        rSig1 = row[cM['COMP_ID_1']] + self.__sC + row[cM['ASYM_ID_1']] + self.__sC + row[cM['SEQ_NUM_1']] + self.__sC + row[cM['INS_CODE_1']]
        rSig2 = row[cM['COMP_ID_2']] + self.__sC + row[cM['ASYM_ID_2']] + self.__sC + row[cM['SEQ_NUM_2']] + self.__sC + row[cM['INS_CODE_2']]
        return (rSig1,rSig2)

    def getConnect(self):
        cCat=DataCategory("pdbx_geom_bond")
        cCat.appendAttribute("atom_site_id_1")
        cCat.appendAttribute("atom_signature_1")
        cCat.appendAttribute("atom_site_id_2")
        cCat.appendAttribute("atom_signature_2")
        cCat.appendAttribute("symmetry_2")
        cCat.appendAttribute("value_order")
        #cCat.appendAttribute("model_number")
        #
        rowList=[]
        #
        # Non-standard components internal connectivities --
        #
        for nsCC in self.__nsCcList:
            atD=self.__nsCcDict[nsCC]
            altL=self.__nsCcAltDict[nsCC]
            fields=nsCC.split(self.__sC)
            nsCompId=str(fields[0]).strip()
            #
            if self.__bondTemplates.has_key(nsCompId):
                bT=self.__bondTemplates[nsCompId]
                for b in bT:
                    tAtI=b[1]
                    tAtJ=b[2]
                    tOrder=b[3]
                    tArom=b[4]
                    if tArom.upper()=='Y':
                        bType='AROM'
                    else:
                        bType=tOrder.upper()
                    for alt in altL:
                        atI=tAtI + self.__sC + alt
                        atJ=tAtJ + self.__sC + alt
                        tsigI= atI + self.__sC +  nsCC
                        tsigJ= atJ + self.__sC +  nsCC
                        sigI=tsigI.replace('?','_').replace('.','_')
                        sigJ=tsigJ.replace('?','_').replace('.','_')
                        if atD.has_key(atI) and atD.has_key(atJ):
                            rowList.append((atD[atI], sigI, atD[atJ], sigJ, '1_555',bType))
        #
        # Now do the links
        #
        if (self.__debug):
            logger.info("+DEBUG - __linkCcDict residue keys %r\n" % self.__linkCcDict.keys())

        for link in self.__linkList:
            for mNum in self.__modelList:
                rSig1=link[0] + self.__sC + str(mNum)
                aSig1=link[1]
                symOp1=link[2]
                tsig1=aSig1 + self.__sC + rSig1
                sig1=tsig1.replace('?','_').replace('.','_')
                #
                rSig2=link[3] + self.__sC + str(mNum)
                aSig2=link[4]
                symOp2=link[5]
                tsig2=aSig2 + self.__sC + rSig2
                sig2=tsig2.replace('?','_').replace('.','_')
                #
                try:
                    idx1=self.__linkCcDict[rSig1][aSig1]
                    idx2=self.__linkCcDict[rSig2][aSig2]
                    if symOp1 == '1_555':
                        rowList.append(( idx1 , sig1, idx2 , sig2, symOp2, 'SING'))
                    elif symOp2 == '1_555':
                        rowList.append(( idx2 , sig2, idx1 , sig1, symOp1, 'SING'))
                    else:
                        logger.info("No link members in asymmetric unit %s %s %s %s\n" % (rSig1,aSig1,rSig2,aSig2))
                except Exception as e:
                    logger.info("Failing link lookup - %r model %r numModels %d\n" % (link,mNum,len(self.__modelList)))
                    logger.exception("Failing with %s" % str(e))
                    continue

        #
        #sorted(rowList,key=itemgetter(0,2))
        #
        # Fill in the data for the return category
        #
        #        for row in sorted(rowList,key=itemgetter(0,2),cmp=numeric_compare):
        for row in sorted(rowList,key=itemgetter(0),cmp=numeric_compare):
            cCat.append(row)
        #
        self.__dBlock.append(cCat)
        return cCat
        #


    def getConnectIndexOnly(self):
        cCat=DataCategory("pdbx_geom_bond")
        cCat.appendAttribute("atom_site_id_1")
        cCat.appendAttribute("atom_site_id_2")
        cCat.appendAttribute("symmetry_2")
        cCat.appendAttribute("value_order")
        #cCat.appendAttribute("model_number")
        #
        rowList=[]
        #
        # Non-standard components internal connectivities --
        #
        for nsCC in self.__nsCcList:
            atD=self.__nsCcDict[nsCC]
            altL=self.__nsCcAltDict[nsCC]
            fields=nsCC.split(self.__sC)
            nsCompId=str(fields[0]).strip()
            #
            if self.__bondTemplates.has_key(nsCompId):
                bT=self.__bondTemplates[nsCompId]
                for b in bT:
                    tAtI=b[1]
                    tAtJ=b[2]
                    tOrder=b[3]
                    tArom=b[4]
                    if tArom.upper()=='Y':
                        bType='AROM'
                    else:
                        bType=tOrder.upper()
                    for alt in altL:
                        atI=tAtI + self.__sC + alt
                        atJ=tAtJ + self.__sC + alt
                        if atD.has_key(atI) and atD.has_key(atJ):
                            rowList.append((atD[atI], atD[atJ],'1_555',bType))
        #
        # Now do the links
        #
        if (self.__debug):
            logger.info("+DEBUG - __linkCcDict residue keys %r\n" % self.__linkCcDict.keys())

        for link in self.__linkList:
            for mNum in self.__modelList:
                rSig1=link[0] + self.__sC + str(mNum)
                aSig1=link[1]
                symOp1=link[2]

                rSig2=link[3] + self.__sC + str(mNum)
                aSig2=link[4]
                symOp2=link[5]
                try:
                    idx1=self.__linkCcDict[rSig1][aSig1]
                    idx2=self.__linkCcDict[rSig2][aSig2]
                    if symOp1 == '1_555':
                        rowList.append(( idx1 , idx2 , symOp2, 'SING'))
                    elif symOp2 == '1_555':
                        rowList.append(( idx2 , idx1 , symOp1, 'SING'))
                    else:
                        logger.info("No link members in asymmetric unit %s %s %s %s\n" % (rSig1,aSig1,rSig2,aSig2))
                except Exception as e:
                    logger.info("Failing link lookup - %r model %r numModels %d\n" % (link,mNum,len(self.__modelList)))
                    logger.exception("Failing with %s" % str(e))
                    continue

        #
        sorted(rowList,key=itemgetter(0,1))
        #
        # Fill in the data for the return category
        #
        for row in rowList:
            cCat.append(row)
        #
        self.__dBlock.append(cCat)
        return cCat
        #

    def getLinkData(self,catName='struct_conn'):
        """Return a list linkages from the struct_conn category.
        """
        self.__linkCcList=[]
        self.__linkCcSigList=[]
        self.__linkList=[]

        catObj=self.__dBlock.getObj(catName)
        if catObj is None:
            return

        nRows=catObj.getRowCount()

        itDict={}
        itNameList=catObj.getItemNameList()
        for idxIt,itName in enumerate(itNameList):
            itDict[itName]=idxIt
        #
        cM={}
        for k,v in self.__structConnMap.items():
            if itDict.has_key(v):
                cM[k]=itDict[v]
            else:
                cM[k]=-1
        #

        rowList=catObj.getRowList()
        for row in rowList:
            #
            if ( not ( row[cM['CONN_TYPE']].startswith('covale')  or row[cM['CONN_TYPE']].startswith('disulf')  or row[cM['CONN_TYPE']].startswith('metal'))):
                continue
            (rSig1,rSig2)=self.__getCompSignatureStructConn(row,cM)
            if rSig1 not in self.__linkCcSigList:
                self.__linkCcSigList.append(rSig1)

            if rSig2 not in self.__linkCcSigList:
                self.__linkCcSigList.append(rSig2)

            if (row[cM['COMP_ID_1']] not in self.__linkCcList):
                self.__linkCcList.append(row[cM['COMP_ID_1']])

            if (row[cM['COMP_ID_2']] not in self.__linkCcList):
                self.__linkCcList.append(row[cM['COMP_ID_2']])

            if (row[cM['ALT_LOC_1']] == '?'):
                altLoc1='.'
            else:
                altLoc1=row[cM['ALT_LOC_1']]

            if (row[cM['ALT_LOC_2']] == '?'):
                altLoc2='.'
            else:
                altLoc2=row[cM['ALT_LOC_2']]

            aSig1=row[cM['ATOM_NAME_1']] + self.__sC + altLoc1
            aSig2=row[cM['ATOM_NAME_2']] + self.__sC + altLoc2

            symOp1=row[cM['SYM_OP_1']]
            symOp2=row[cM['SYM_OP_2']]

            self.__linkList.append((rSig1,aSig1,symOp1, rSig2,aSig2,symOp2 ))

        #
        if (self.__debug):
            logger.info("\n+DEBUG - __linkCcSigList %r\n" % self.__linkCcSigList)
            logger.info("\n+DEBUG - __linkCcList %r\n" % self.__linkList)






##
# File: PdbxPrdCcIo.py
# Date: 22-Jan-2013  John Westbrook
#
# Update:
#
##
"""
Collected methods for accessing BIRD PRD chemical component analog definitions.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"

import sys
import os
import logging
logger = logging.getLogger(__name__)

from mmcif_utils.style.PdbxStyleIoUtil import PdbxStyleIoUtil
from mmcif_utils.style.ChemCompCategoryStyle import ChemCompCategoryStyle


class PdbxPrdCcIo(PdbxStyleIoUtil):
    ''' Methods for reading BIRD PRD and Family definitions subject to style details.

    '''

    def __init__(self, verbose=True, log=sys.stderr):
        super(PdbxPrdCcIo, self).__init__(styleObject=ChemCompCategoryStyle(), verbose=verbose, log=log)

        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        #
        # self.__dBlock=None
        #
        self.__topCachePath = None
        self.__prdCcId = None
        self.__filePath = None

    def makeDefinitionPathList(self):
        """ Return the list of definition file paths in the current repository.

            List is ordered in increasing PRD ID numerical code.
        """
        pathList = []
        sd = {}
        for root, dirs, files in os.walk(self.__topCachePath, topdown=False):
            if "REMOVE" in root:
                continue
            for name in files:
                if name.startswith("PRDCC_") and name.endswith(".cif") and len(name) <= 16:
                    pth = os.path.join(root, name)
                    sd[int(name[6:-4])] = pth
        #
        for k in sorted(sd.keys()):
            pathList.append(sd[k])
        #
        return pathList

    def makeDefinitionIdList(self):
        """ Return the list of definition identifiers in the current repository.

            List is ordered in increasing PRD ID numerical code.
        """
        idList = []
        sd = {}
        for root, dirs, files in os.walk(self.__topCachePath, topdown=False):
            if "REMOVE" in root:
                continue
            for name in files:
                if name.startswith("PRDCC_") and name.endswith(".cif") and len(name) <= 16:
                    sd[int(name[6:-4])] = name[6:-4]
        #
        for k in sorted(sd.keys()):
            idList.append(sd[k])
        #
        return idList

    def setCachePath(self, topCachePath='/data/components/prdcc-v3'):
        self.__topCachePath = topCachePath

    def setPrdCcId(self, prdCcId):
        """ Set the identifier for the target definition.   The internal target file path
            is set to the definition file stored in the organization of CVS repository if this exists.

            returns True for success or False otherwise.
        """
        self.__prdCcId = str(prdCcId).upper()
        self.__filePath = os.path.join(self.__topCachePath, self.__prdCcId[-1], self.__prdCcId + '.cif')
        #
        if self.readFile(self.__filePath):
            return self.setContainer(containerName=self.__prdCcId)
        else:
            return False

    def setFilePath(self, filePath, prdCcId=None, appendMode=True):
        """ Specify the file path for the target definition  and optionally provide an identifier
            for the data section within the file.
        """
        self.__filePath = filePath
        self.__prdCcId = str(prdCcId).upper()
        if self.readFile(self.__filePath, appendMode=appendMode):
            if self.__prdCcId is not None:
                return self.setContainer(containerName=self.__prdCcId)
            else:
                return self.setContainer(containerIndex=0)
        else:
            return False

    def getPrdCc(self):
        """
            Check for a valid current definition selection.

            Returns True for success or False otherwise.
        """
        return (self.getCurrentContainerId() is not None)

    def getCategory(self, catName='pdbx_reference_entity'):
        return self.getItemDictList(catName)

    def complyStyle(self):
        return self.testStyleComplete(self.__lfh)

    def update(self, catName, attributeName, value, iRow=0):
        #
        return self.updateAttribute(catName, attributeName, value, iRow=iRow)

    def write(self, filePath):
        return self.writeFile(filePath)
        #

    def setBlock(self, blockId):
        return self.setContainer(containerName=blockId)

    def newBlock(self, blockId):
        return self.newContainer(containerName=str(blockId).upper())



##
# File: PdbxPrdUtils.py
# Date: 28-Nov-2012
#
# Update:
#
# 21-Jan-2013 jdw add options to update schema table with materialized 3-letter-code sequences.
#  1-May-2013 jdw add method for general status and audit update.
#
##
"""
A collection of utilities methods for BIRD PRD and family definitions.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"

import sys
import time
import os
import traceback

import logging
logger = logging.getLogger(__name__)

import itertools

from mmcif_utils.bird.PdbxPrd import *


class PdbxPrdUtils(object):
    """ Utilities methods for BIRD PRD and family definitions.

    """

    def __init__(self, PdbxPrdIoObj=None, verbose=True, log=sys.stderr):
        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        self.__pObj = PdbxPrdIoObj

    def getFamilyMembers(self):
        """ Return of a dictionary of PRD family members for each family defintion within the
            current PRD container collection.

            Return dictionary contains a list of member Prd Id codes for each family -
                s[familyId] = [prdId1,prdId2,...]
        """
        _mId = "(" + self.__class__.__name__ + "." + sys._getframe().f_code.co_name + ")"
        fD = {}
        for prdIdC in self.__pObj.getContainerNameList():
            self.__pObj.setContainer(prdIdC)
            id = self.__pObj.getCurrentContainerId()
            if id == prdIdC:
                dL = self.__pObj.getAttribDictList(catName='pdbx_reference_molecule_list')
                for d in dL:
                    rOb = PdbxReferenceMoleculeList(d, self.__verbose, self.__lfh)
                    prdId = rOb.getPrdId()
                    familyId = rOb.getFamilyPrdId()
                    if familyId not in fD:
                        fD[familyId] = []
                    if familyId == id:
                        fD[familyId].append(prdId)
                    else:
                        logger.info("+ERROR 0 - %s container %s id mismatch %s\n" % (_mId, id, familyId))
        return fD

    def getChemCompIds(self):
        """ Return of a dictionary of corresponding chemical component identifiers

            Return dictionary contains the status codes organized by prdId -
                s[prdId] = CCID or PRDCC_ID
        """
        _mId = "(" + self.__class__.__name__ + "." + sys._getframe().f_code.co_name + ")"
        pD = {}
        for prdIdC in self.__pObj.getContainerNameList():
            self.__pObj.setContainer(prdIdC)
            id = self.__pObj.getCurrentContainerId()
            if id == prdIdC:
                dL = self.__pObj.getAttribDictList(catName='pdbx_reference_molecule')
                for d in dL:
                    rOb = PdbxReferenceMolecule(d, self.__verbose, self.__lfh)
                    repType = rOb.getRepresentAs()
                    ccId = rOb.getChemCompId()
                    if ccId is not None and ccId not in ['.', '?']:
                        pD[id] = ccId
                    elif repType is not None and repType == 'polymer':
                        pD[id] = "PRDCC_" + id[4:]
                    else:
                        logger.info("+ERROR 0 - %s container %s id representation error\n" % (_mId, id))
            else:
                logger.info("+ERROR 0 - %s container %s id mismatch %s\n" % (_mId, id, pId))
        return pD

    def getStatus(self):
        """ Return of a dictionary of defintion status for the current PRD container collection.

            Return dictionary contains the status codes organized by prdId -
                s[prdId] = REL, HOLD, ...
        """
        _mId = "(" + self.__class__.__name__ + "." + sys._getframe().f_code.co_name + ")"
        sD = {}
        for prdIdC in self.__pObj.getContainerNameList():
            self.__pObj.setContainer(prdIdC)
            id = self.__pObj.getCurrentContainerId()
            if id == prdIdC:
                dL = self.__pObj.getAttribDictList(catName='pdbx_reference_molecule')
                for d in dL:
                    rOb = PdbxReferenceMolecule(d, self.__verbose, self.__lfh)
                    status = rOb.getReleaseStatus()
                    pId = rOb.getId()
                    if pId == id:
                        sD[pId] = status
                    else:
                        logger.info("+ERROR 0 - %s container %s id mismatch %s\n" % (_mId, id, pId))
        return sD

    def suppressComponentsFromFile(self, filePath=None):
        """ Suppress values of _pdbx_reference_molecule.chem_comp_id that are listed in the
            input file.

        """
        _mId = "(" + self.__class__.__name__ + "." + sys._getframe().f_code.co_name + ")"
        #
        if not os.access(filePath, os.R_OK):
            logger.info("+ERROR 0 - %s cannot open reference id list %s\n" % (_mId, filePath))
        refD = {}
        ifh = open(filePath, 'r')
        for line in ifh.readlines():
            id = str(line[:-1])
            refD[id] = id
        ifh.close()
        #
        self.__suppressComponents(refD=refD)

    def suppressComponentsFromList(self, idList=None):
        """ Suppress values of _pdbx_reference_molecule.chem_comp_id that are in the input id list.
        """
        #
        sL = idList if idList is not None else []
        refD = {}
        for id in sL:
            refD[id] = id
        self.__suppressComponents(refD=refD)

    def __suppressComponents(self, refD={}):
        """ Suppress values of _pdbx_reference_molecule.chem_comp_id that are keys in the
            input dictionary.
        """
        #
        _mId = "(" + self.__class__.__name__ + "." + sys._getframe().f_code.co_name + ")"
        for prdIdC in self.__pObj.getContainerNameList():
            self.__pObj.setContainer(prdIdC)
            id = self.__pObj.getCurrentContainerId()
            if id == prdIdC:
                myCatName = 'pdbx_reference_molecule'
                dL = self.__pObj.getAttribDictList(catName=myCatName)
                for iRow, rowD in enumerate(dL):
                    rOb = PdbxReferenceMolecule(rowD, self.__verbose, self.__lfh)
                    ccId = rOb.getChemCompId()
                    if ccId in refD:
                        rOb.setChemCompId(None)
                        self.__pObj.updateRowByAttribute(rowD, myCatName, iRow=iRow)
                        if (self.__verbose):
                            logger.info("+INFO - %s suppressing component id %s in %s\n" % (_mId, ccId, id))

    def setReleaseAndAudit(self, relDate="YYYY-MM-DD"):
        """
        """
        _mId = "(" + self.__class__.__name__ + "." + sys._getframe().f_code.co_name + ")"
        for prdIdC in self.__pObj.getContainerNameList():
            self.__pObj.setContainer(prdIdC)
            id = self.__pObj.getCurrentContainerId()
            if id == prdIdC:
                myCatName = 'pdbx_reference_molecule'
                dL = self.__pObj.getAttribDictList(catName=myCatName)
                for iRow, rowD in enumerate(dL):
                    rOb = PdbxReferenceMolecule(rowD, self.__verbose, self.__lfh)
                    ok = rOb.setReleaseStatus('REL')
                    self.__pObj.updateRowByAttribute(rowD, myCatName, iRow=iRow)

                aFlag = False
                # first check for an existing record
                myCatName = 'pdbx_prd_audit'
                dL = self.__pObj.getAttribDictList(catName=myCatName)
                for iRow, rowD in enumerate(dL):
                    rOb = PdbxPrdAudit(rowD, self.__verbose, self.__lfh)
                    actionType = rOb.getActionType()
                    if actionType == "Initial release":
                        aFlag = True
                        ok1 = rOb.setDate(relDate)
                        ok2 = self.__pObj.updateRowByAttribute(rowD, myCatName, iRow=iRow)
                        if (self.__verbose):
                            logger.info("+INFO - %s id %s setting date %s in %r %r\n" % (_mId, id, relDate, ok1, ok2))

                if not aFlag:
                    rowD = {}
                    rOb = PdbxPrdAudit(rowD, self.__verbose, self.__lfh)
                    ok = rOb.setActionType("Initial release")
                    ok = rOb.setId(id)
                    ok = rOb.setDate(relDate)
                    ok = rOb.setProcessingSite("RCSB")
                    #
                    self.__pObj.appendRowByAttribute(rowD, myCatName)

    def setStatusAndAudit(self, statusCode='HOLD', updateDate="YYYY-MM-DD", site="RCSB", annotator=None, details=None):
        """
        """
        _mId = "(" + self.__class__.__name__ + "." + sys._getframe().f_code.co_name + ")"
        for prdIdC in self.__pObj.getContainerNameList():
            self.__pObj.setContainer(prdIdC)
            id = self.__pObj.getCurrentContainerId()
            if id == prdIdC:
                myCatName = 'pdbx_reference_molecule'
                dL = self.__pObj.getAttribDictList(catName=myCatName)
                for iRow, rowD in enumerate(dL):
                    rOb = PdbxReferenceMolecule(rowD, self.__verbose, self.__lfh)
                    ok = rOb.setReleaseStatus(statusCode)
                    self.__pObj.updateRowByAttribute(rowD, myCatName, iRow=iRow)

                myCatName = 'pdbx_prd_audit'
                dL = self.__pObj.getAttribDictList(catName=myCatName)

                rowD = {}
                rOb = PdbxPrdAudit(rowD, self.__verbose, self.__lfh)
                ok = rOb.setActionType("Modify audit")
                ok = rOb.setId(id)
                ok = rOb.setDate(updateDate)
                ok = rOb.setProcessingSite(site)
                if annotator is not None:
                    ok = rOb.setAnnotator(annotator)
                if details is not None:
                    ok = rOb.setDetails(details)
                #
                self.__pObj.appendRowByAttribute(rowD, myCatName)

    def getReferencedComponents(self):
        """ Get referenced chemical components within the current PRD container collection.

            Return a dictionary or components containing the list -
                d[prdId] = [
        """
        _mId = "(" + self.__class__.__name__ + "." + sys._getframe().f_code.co_name + ")"
        compD = {}
        for prdIdC in self.__pObj.getContainerNameList():
            self.__pObj.setContainer(prdIdC)
            id = self.__pObj.getCurrentContainerId()
            if id == prdIdC:
                dL = self.__pObj.getAttribDictList(catName='pdbx_reference_entity_poly_seq')
                for d in dL:
                    rOb = PdbxReferenceEntityPolySeq(d, self.__verbose, self.__lfh)
                    pId = rOb.getId()
                    cc = rOb.getComponentId()
                    if pId == id:
                        if pId not in compD:
                            compD[pId] = []
                        compD[pId].append(cc)
                    else:
                        logger.info("+ERROR 1 - %s container %s id mismatch %s\n" % (_mId, id, pId))

                dL = self.__pObj.getAttribDictList(catName='pdbx_reference_entity_nonpoly')
                for d in dL:
                    rOb = PdbxReferenceEntityNonPoly(d, self.__verbose, self.__lfh)
                    pId = rOb.getId()
                    cc = rOb.getComponentId()
                    if pId == id:
                        if pId not in compD:
                            compD[pId] = []
                        compD[pId].append(cc)
                    else:
                        logger.info("+ERROR 2 - %s container %r id mismatch %r\n" % (_mId, id, pId))

                dL = self.__pObj.getAttribDictList(catName='pdbx_reference_entity_link')
                for d in dL:
                    rOb = PdbxReferenceEntityLink(d, self.__verbose, self.__lfh)
                    pId = rOb.getId()
                    cc1 = rOb.getComponentId1()
                    cc2 = rOb.getComponentId1()
                    if pId == id:
                        if pId not in compD:
                            compD[pId] = []
                        compD[pId].append(cc1)
                        compD[pId].append(cc2)
                    else:
                        logger.info("+ERROR 3 - %s container %s id mismatch %s\n" % (_mId, id, pId))

                dL = self.__pObj.getAttribDictList(catName='pdbx_reference_entity_poly_link')
                for d in dL:
                    rOb = PdbxReferenceEntityPolyLink(d, self.__verbose, self.__lfh)
                    pId = rOb.getId()
                    cc1 = rOb.getComponentId1()
                    cc2 = rOb.getComponentId1()
                    if pId == id:
                        if pId not in compD:
                            compD[pId] = []
                        compD[pId].append(cc1)
                        compD[pId].append(cc2)
                    else:
                        logger.info("+ERROR 4 - %s container %s id mismatch %s\n" % (_mId, id, pId))

                dL = self.__pObj.getAttribDictList(catName='pdbx_reference_entity_nonpoly_link')
                for d in dL:
                    rOb = PdbxReferenceEntityNonPolyLink(d, self.__verbose, self.__lfh)
                    pId = rOb.getId()
                    cc1 = rOb.getComponentId1()
                    cc2 = rOb.getComponentId1()
                    if pId == id:
                        if pId not in compD:
                            compD[pId] = []
                        compD[pId].append(cc1)
                        compD[pId].append(cc2)
                    else:
                        logger.info("+ERROR 5 - %s container %s id mismatch %s\n" % (_mId, id, pId))

        rD = {}
        for id in sorted(compD.keys()):
            t = set([cc for cc in compD[id] if cc not in [None, '.', '?']])
            if len(t) > 0:
                rD[id] = list(t)

        return rD

    def getComponentSequences(self, addCategory=False):
        """ Get component sequences (3-letter-codes) as a list for each polymer reference entity in
            in the current definition container list.

            Return a dictionary of components containing the list -
                d[prdId][refEntityId]=[res1,res2,...]
        """

        seqD = {}
        for prdIdC in self.__pObj.getContainerNameList():
            self.__pObj.setContainer(prdIdC)
            id = self.__pObj.getCurrentContainerId()
            if id == prdIdC:
                dL = self.__pObj.getAttribDictList(catName='pdbx_reference_entity_poly_seq')
                refD = {}
                for d in dL:
                    rOb = PdbxReferenceEntityPolySeq(d, self.__verbose, self.__lfh)
                    pid = rOb.getId()
                    refId = rOb.getReferenceEntityId()
                    compId = rOb.getComponentId()
                    num = int(rOb.getSequenceNumber())
                    het = rOb.getHeterogeneityFlag()
                    #
                    if refId not in refD:
                        refD[refId] = {}

                    if num not in refD[refId]:
                        refD[refId][num] = []

                    refD[refId][num].append((num, compId))

                seqD[prdIdC] = refD
            else:
                logger.info("+ERROR - container %s id mismatch %s\n" % (id, prdIdC))
        #
        rD = {}
        for id in sorted(seqD.keys()):
            if id not in rD:
                rD[id] = {}
            for refId, sD in seqD[id].items():

                vL = self.__makeVariantCombinations(seqVarD=sD)
                if (len(vL) > 0):
                    if (self.__debug):
                        logger.info("%s %s variants %d list: %s\n" % (id, refId, len(vL), vL))
                    tmpLoL = []
                    for ii, vtup in enumerate(vL):
                        tmpL = []
                        for ii in sorted(sD.keys()):
                            if len(sD[ii]) > 1:
                                choice = list(set(sD[ii]) & set(vtup))
                            else:
                                choice = sD[ii]
                            tmpL.append(choice[0][1])
                        tmpLoL.append(tmpL)
                    rD[id][refId] = tmpLoL
                else:
                    tmpL = []
                    for ii in sorted(sD.keys()):
                        choice = sD[ii]
                        tmpL.append(choice[0][1])
                    rD[id][refId] = [tmpL]

        # Create a new category to hold the sequence data --
        #
        if (addCategory):
            for id in sorted(rD.keys()):
                ok = self.__pObj.setContainer(containerName=id)
                if not ok:
                    logger.info("+PrdUtility(getComponentSequences) container lookup failed %s\n" % id)
                self.__pObj.newCategory(catName='pdbx_reference_entity_sequence_list')
                iRow = 0
                for refId, seq3L in rD[id].items():
                    for ii, seq3 in enumerate(seq3L, start=1):
                        tD = {}
                        tD['prd_id'] = id
                        tD['ref_entity_id'] = refId
                        tD['v_id'] = str(ii)
                        tD['three_letter_codes'] = ' '.join(seq3)
                        self.__pObj.updateRowByAttribute(rowAttribDict=tD, catName='pdbx_reference_entity_sequence_list', iRow=iRow)
                        iRow += 1
                        if self.__debug:
                            logger.info("+PrdUtility(getComponentSequences) row dict: %r\n" % tD.items())
                        #
        return rD

    def __makeVariantCombinations(self, seqVarD={}):
        #
        varLofL = []
        #
        for k, vL in seqVarD.items():
            if len(vL) > 1:
                varLofL.append(vL)
        #
        vL = []
        if (len(varLofL) > 0):
            if (self.__debug):
                logger.info("+PrdUtility(__makeVariantCombinations) List - %r\n" % varLofL)
            vL = list(itertools.product(*varLofL))
            if (self.__debug):
                logger.info("+PrdUtility(__makeVariantCombinations) Product list: %r\n" % vL)

        return vL



##
# File: PdbxVersion.py
# Date: 20-May-2011  John Westbrook
#
# Updated:
#  23-Oct-2012 jdw  Update path and reorganize
#
##
"""
A collection of classes supporting pdbx_version data.

"""
__docformat__ = "restructuredtext en"
__author__    = "John Westbrook"
__email__     = "john.westbrook@rcsb.org"
__license__   = "Apache 2.0"
__version__   = "V0.01"


import sys,time,os,traceback
from mmcif.io.PdbxReader import PdbxReader
#from pdbx-v2.reader.PdbxContainers import *

class PdbxVersionCategoryDefinition(object):


    _categoryInfo = [('pdbx_version', 'table'),
                     ]

    _cDict={
        'pdbx_version' : [
            ('_pdbx_version.entry_id','%s','str',''),
            ('_pdbx_version.revision_date','%s','str',''),
            ('_pdbx_version.major_version','%s','str',''),
            ('_pdbx_version.minor_version','%s','str',''),
            ('_pdbx_version.revision_type','%s','str',''),
            ('_pdbx_version.details','%s','str','')
            ]
        }

class PdbxVersionReader(object):
    ''' Accessor methods for pdbx version data.

    '''
    def __init__(self,verbose=True,log=sys.stderr):
        self.__verbose=verbose
        self.__debug=False
        self.__lfh=log
        self.__dBlock=None
        self.__topCachePath=None
        self.__filePath=None
        self.__defaultBlockName='PDBX_VERSION'
        #
    def getBlockId(self):
        try:
            if self.__dBlock is not None:
                return self.__dBlock.getName()
            else:
                False
        except Exception as e:
            return False


    def setFilePath(self,filePath):
        """ Specify the file path for the target file.
        """
        try:
            self.__filePath=filePath
            if (not os.access(self.__filePath,os.R_OK)):
                if (self.__verbose):
                    logger.info("+ERROR- PdbxVersionReader.setFilePath() Missing file %s\n" % self.__filePath)
                return False
            else:
                if (self.__verbose):
                    logger.info("+PdbxVersionReader.setFilePath() file path %s\n" % self.__filePath)
            return True
        except Exception as e:
            if (self.__verbose):
                logger.info("+ERROR- PdbxVersionReader.setFilePath() Missing file %s\n" % self.__filePath)
            return False

    def getVersionData(self):
        """   Get the data -

        """
        try:
            block=self.__getDataBlock(self.__filePath,self.__defaultBlockName)
            return self.__setDataBlock(block)

        except Exception as e:
            logger.exception("Failing with %s" % str(e))
            return False

    def getCategory(self,catName='pdbx_version'):
        return self.__getDictList(catName=catName)



    def __getDataBlock(self,filePath,blockId=None):
        """ Worker method to read the version data file and set the target datablock.
            If no blockId is provided return the first data block.
        """
        try:
            ifh=open(filePath,'r')
            myBlockList=[]
            pRd=PdbxReader(ifh)
            pRd.read(myBlockList)
            ifh.close()
            if (blockId is not None):
                for block in myBlockList:
                    if (block.getType() == 'data' and block.getName() == blockId):
                        if (self.__debug):
                            block.printIt(self.__lfh)
                        return block
            else:
                for block in myBlockList:
                    if (block.getType() == 'data'):
                        if (self.__debug):
                            block.printIt(self.__lfh)
                        return block

            return None
        except Exception as e:
            logger.exception("Failing with %s" % str(e))
            return None

    def __setDataBlock(self,dataBlock=None):
        """ Assigns the input data block as the active internal data block containing the
            target version data.
        """
        ok=False
        try:
            if dataBlock.getType() == 'data':
                self.__dBlock=dataBlock
                ok=True
            else:
                self.__dBlock=None
        except Exception as e:
            pass

        return ok


    def getCategoryList(self):
        if self.__dBlock is not None:
            return self.__dBlock.getObjNameList()
        else:
            return []



    def __getDictList(self,catName='pdbx_version'):
        """Return the input category as a list of dictionaries where the dictionaries
           represent the row with full item names as dictionary keys.
        """
        #
        dList=[]
        #
        # Get category object - from current data block
        #
        itTupList= PdbxVersionCategoryDefinition._cDict[catName]
        catObj=self.__dBlock.getObj(catName)
        if catObj is None:
            return dList

        nRows=catObj.getRowCount()
        #
        # Get column name index.
        #
        itDict={}
        itNameList=catObj.getItemNameList()
        for idxIt,itName in enumerate(itNameList):
            itDict[itName]=idxIt
        #
        # Find the mapping to the local category definition
        #
        colDict={}
        #
        for ii,itTup in enumerate(itTupList):
            if itDict.has_key(itTup[0]):
                colDict[itTup[0]] = itDict[itTup[0]]
        #
        rowList=catObj.getRowList()
        for row in rowList:
            tD={}
            for k,v in colDict.items():
                tD[k]=row[v]
            dList.append(tD)

        return dList

    def __getDataList(self,catName='pdbx_reference_entity'):
        """Return  a list of data from the input category including
           data types and default value replacement.

           For list representing each row is column ordered according to the internal
           data structure PdbxVersionCategoryDefinition._cDict[catName]

        """
        dataList=[]
        itTupList= PdbxVersionCategoryDefinition._cDict[catName]
        catObj=self.__dBlock.getObj(catName)
        if (catObj is None):
            return dataList

        nRows=catObj.getRowCount()

        itDict={}
        itNameList=catObj.getItemNameList()
        for idxIt,itName in enumerate(itNameList):
            itDict[itName]=idxIt
        #
        colTupList=[]
        # (column index of data or -1, type name, [default value]  )
        for ii,itTup in enumerate(itTupList):
            if itDict.has_key(itTup[0]):
                colTupList.append( (itDict[itTup[0]], itTup[2], itTup[3] ))
            else:
                colTupList.append( (-1, itTup[2], itTup[3]) )
        #
        rowList=catObj.getRowList()

        for row in rowList:
            uR=[]
            for cTup in colTupList:

                if cTup[0] < 0:
                    uR.append(self.__applyType(cTup[1],cTup[2],cTup[2]) )
                else:
                    uR.append(self.__applyType(cTup[1],cTup[2],row[cTup[0]]))

            dataList.append(uR)

        return dataList

    def __applyType(self,type,default,val):
        """Apply type conversion to the input value and assign default values to
           missing values.
        """
        tval = val
        if (val is None):
            tval=default
        if (isinstance(tval,str) and (len(tval)<1 or tval == '.' or tval == '?')):
            tval=default

        if type == "int":
            return int(str(tval))
        elif type == "float":
            return float(str(tval))
        elif type == "str":
            return str(tval)
        else:
            return tval






##
# File: PdbxGeometryReportCategoryStyle.py
# Date: 1-Jan-2014
#
# Updates:
#   5-Mar-2018  jdw Py2-Py3 and refactor for Python Packaging
#
##
"""
Report style details for PDBx geometry validation data categories.

"""
from __future__ import absolute_import
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"


from mmcif_utils.style.PdbxCategoryStyleBase import PdbxCategoryStyleBase


class PdbxGeometryReportCategoryStyle(PdbxCategoryStyleBase):
    _styleId = "PDBX_GEOMETRY_REPORT_V1"
    _categoryInfo = [
        ('pdbx_validate_close_contact', 'table'),
        ('pdbx_validate_symm_contact', 'table'),
        ('pdbx_validate_rmsd_bond', 'table'),
        ('pdbx_validate_rmsd_angle', 'table'),
        ('pdbx_validate_torsion', 'table'),
        ('pdbx_validate_peptide_omega', 'table'),
        ('pdbx_validate_main_chain_plane', 'table'),
        ('pdbx_validate_polymer_linkage', 'table'),
        ('pdbx_validate_chiral', 'table'),
    ]
    _cDict = {
        'pdbx_validate_close_contact': [
            ('_pdbx_validate_close_contact.id', '%s', 'str', ''),
            ('_pdbx_validate_close_contact.PDB_model_num', '%s', 'str', ''),
            ('_pdbx_validate_close_contact.auth_atom_id_1', '%s', 'str', ''),
            ('_pdbx_validate_close_contact.auth_asym_id_1', '%s', 'str', ''),
            ('_pdbx_validate_close_contact.auth_comp_id_1', '%s', 'str', ''),
            ('_pdbx_validate_close_contact.auth_seq_id_1', '%s', 'str', ''),
            ('_pdbx_validate_close_contact.PDB_ins_code_1', '%s', 'str', ''),
            ('_pdbx_validate_close_contact.label_alt_id_1', '%s', 'str', ''),
            ('_pdbx_validate_close_contact.auth_atom_id_2', '%s', 'str', ''),
            ('_pdbx_validate_close_contact.auth_asym_id_2', '%s', 'str', ''),
            ('_pdbx_validate_close_contact.auth_comp_id_2', '%s', 'str', ''),
            ('_pdbx_validate_close_contact.auth_seq_id_2', '%s', 'str', ''),
            ('_pdbx_validate_close_contact.PDB_ins_code_2', '%s', 'str', ''),
            ('_pdbx_validate_close_contact.label_alt_id_2', '%s', 'str', ''),
            ('_pdbx_validate_close_contact.dist', '%s', 'str', '')
        ],
        'pdbx_validate_symm_contact': [
            ('_pdbx_validate_symm_contact.id', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.PDB_model_num', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.auth_atom_id_1', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.auth_asym_id_1', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.auth_comp_id_1', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.auth_seq_id_1', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.PDB_ins_code_1', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.label_alt_id_1', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.site_symmetry_1', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.auth_atom_id_2', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.auth_asym_id_2', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.auth_comp_id_2', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.auth_seq_id_2', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.PDB_ins_code_2', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.label_alt_id_2', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.site_symmetry_2', '%s', 'str', ''),
            ('_pdbx_validate_symm_contact.dist', '%s', 'str', '')
        ],
        'pdbx_validate_rmsd_bond': [
            ('_pdbx_validate_rmsd_bond.id', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.PDB_model_num', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.auth_atom_id_1', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.auth_asym_id_1', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.auth_comp_id_1', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.auth_seq_id_1', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.PDB_ins_code_1', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.label_alt_id_1', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.auth_atom_id_2', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.auth_asym_id_2', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.auth_comp_id_2', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.auth_seq_id_2', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.PDB_ins_code_2', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.label_alt_id_2', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.bond_value', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.bond_target_value', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.bond_deviation', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.bond_standard_deviation', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_bond.linker_flag', '%s', 'str', '')
        ],
        'pdbx_validate_rmsd_angle': [
            ('_pdbx_validate_rmsd_angle.id', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.PDB_model_num', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.auth_atom_id_1', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.auth_asym_id_1', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.auth_comp_id_1', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.auth_seq_id_1', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.PDB_ins_code_1', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.label_alt_id_1', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.auth_atom_id_2', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.auth_asym_id_2', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.auth_comp_id_2', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.auth_seq_id_2', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.PDB_ins_code_2', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.label_alt_id_2', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.auth_atom_id_3', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.auth_asym_id_3', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.auth_comp_id_3', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.auth_seq_id_3', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.PDB_ins_code_3', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.label_alt_id_3', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.angle_value', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.angle_target_value', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.angle_deviation', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.angle_standard_deviation', '%s', 'str', ''),
            ('_pdbx_validate_rmsd_angle.linker_flag', '%s', 'str', '')
        ],
        'pdbx_validate_torsion': [
            ('_pdbx_validate_torsion.id', '%s', 'str', ''),
            ('_pdbx_validate_torsion.PDB_model_num', '%s', 'str', ''),
            ('_pdbx_validate_torsion.auth_comp_id', '%s', 'str', ''),
            ('_pdbx_validate_torsion.auth_asym_id', '%s', 'str', ''),
            ('_pdbx_validate_torsion.auth_seq_id', '%s', 'str', ''),
            ('_pdbx_validate_torsion.PDB_ins_code', '%s', 'str', ''),
            ('_pdbx_validate_torsion.label_alt_id', '%s', 'str', ''),
            ('_pdbx_validate_torsion.phi', '%s', 'str', ''),
            ('_pdbx_validate_torsion.psi', '%s', 'str', '')
        ],
        'pdbx_validate_peptide_omega': [
            ('_pdbx_validate_peptide_omega.id', '%s', 'str', ''),
            ('_pdbx_validate_peptide_omega.PDB_model_num', '%s', 'str', ''),
            ('_pdbx_validate_peptide_omega.auth_comp_id_1', '%s', 'str', ''),
            ('_pdbx_validate_peptide_omega.auth_asym_id_1', '%s', 'str', ''),
            ('_pdbx_validate_peptide_omega.auth_seq_id_1', '%s', 'str', ''),
            ('_pdbx_validate_peptide_omega.PDB_ins_code_1', '%s', 'str', ''),
            ('_pdbx_validate_peptide_omega.label_alt_id_1', '%s', 'str', ''),
            ('_pdbx_validate_peptide_omega.auth_comp_id_2', '%s', 'str', ''),
            ('_pdbx_validate_peptide_omega.auth_asym_id_2', '%s', 'str', ''),
            ('_pdbx_validate_peptide_omega.auth_seq_id_2', '%s', 'str', ''),
            ('_pdbx_validate_peptide_omega.PDB_ins_code_2', '%s', 'str', ''),
            ('_pdbx_validate_peptide_omega.label_alt_id_2', '%s', 'str', ''),
            ('_pdbx_validate_peptide_omega.omega', '%s', 'str', '')
        ],
        'pdbx_validate_main_chain_plane': [
            ('_pdbx_validate_main_chain_plane.id', '%s', 'str', ''),
            ('_pdbx_validate_main_chain_plane.PDB_model_num', '%s', 'str', ''),
            ('_pdbx_validate_main_chain_plane.auth_comp_id', '%s', 'str', ''),
            ('_pdbx_validate_main_chain_plane.auth_asym_id', '%s', 'str', ''),
            ('_pdbx_validate_main_chain_plane.auth_seq_id', '%s', 'str', ''),
            ('_pdbx_validate_main_chain_plane.PDB_ins_code', '%s', 'str', ''),
            ('_pdbx_validate_main_chain_plane.label_alt_id', '%s', 'str', ''),
            ('_pdbx_validate_main_chain_plane.improper_torsion_angle', '%s', 'str', '')
        ],
        'pdbx_validate_polymer_linkage': [
            ('_pdbx_validate_polymer_linkage.id', '%s', 'str', ''),
            ('_pdbx_validate_polymer_linkage.PDB_model_num', '%s', 'str', ''),
            ('_pdbx_validate_polymer_linkage.auth_atom_id_1', '%s', 'str', ''),
            ('_pdbx_validate_polymer_linkage.auth_asym_id_1', '%s', 'str', ''),
            ('_pdbx_validate_polymer_linkage.auth_comp_id_1', '%s', 'str', ''),
            ('_pdbx_validate_polymer_linkage.auth_seq_id_1', '%s', 'str', ''),
            ('_pdbx_validate_polymer_linkage.PDB_ins_code_1', '%s', 'str', ''),
            ('_pdbx_validate_polymer_linkage.label_alt_id_1', '%s', 'str', ''),
            ('_pdbx_validate_polymer_linkage.auth_atom_id_2', '%s', 'str', ''),
            ('_pdbx_validate_polymer_linkage.auth_asym_id_2', '%s', 'str', ''),
            ('_pdbx_validate_polymer_linkage.auth_comp_id_2', '%s', 'str', ''),
            ('_pdbx_validate_polymer_linkage.auth_seq_id_2', '%s', 'str', ''),
            ('_pdbx_validate_polymer_linkage.PDB_ins_code_2', '%s', 'str', ''),
            ('_pdbx_validate_polymer_linkage.label_alt_id_2', '%s', 'str', ''),
            ('_pdbx_validate_polymer_linkage.dist', '%s', 'str', '')
        ],
        'pdbx_validate_chiral': [
            ('_pdbx_validate_chiral.id', '%s', 'str', ''),
            ('_pdbx_validate_chiral.PDB_model_num', '%s', 'str', ''),
            ('_pdbx_validate_chiral.auth_atom_id', '%s', 'str', ''),
            ('_pdbx_validate_chiral.label_alt_id', '%s', 'str', ''),
            ('_pdbx_validate_chiral.auth_asym_id', '%s', 'str', ''),
            ('_pdbx_validate_chiral.auth_comp_id', '%s', 'str', ''),
            ('_pdbx_validate_chiral.auth_seq_id', '%s', 'str', ''),
            ('_pdbx_validate_chiral.PDB_ins_code', '%s', 'str', ''),
            ('_pdbx_validate_chiral.details', '%s', 'str', '')
        ]
    }
    _excludeList = []
    _suppressList = []
    #

    def __init__(self):
        super(PdbxGeometryReportCategoryStyle, self).__init__(styleId=PdbxGeometryReportCategoryStyle._styleId,
                                                              catFormatL=PdbxGeometryReportCategoryStyle._categoryInfo,
                                                              catItemD=PdbxGeometryReportCategoryStyle._cDict,
                                                              excludeList=PdbxGeometryReportCategoryStyle._excludeList,
                                                              suppressList=PdbxGeometryReportCategoryStyle._suppressList)



##
# File: PdbxMessageIo.py
# Date: 7-Aug-2013  John Westbrook
#
# Update:
#  7-Aug-2013  jdw add 'pdbx_deposition_message_file_reference'
#  16-Aug-2013 rps Updated to accommodate new 'pdbx_deposition_message_status'
#  30-Oct-2013 rps tweak to reflect support for UI feature for classifying messages (e.g. "action required" or "unread")
#  9-Sep-2014  rps add 'pdbx_deposition_message_origcomm_reference'
#  28-Jul-2016 rps read() method updated to accept optional "logtag" parameter
##
"""
Collected methods for accessing message data i/o.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"

import sys

from mmcif_utils.style.PdbxStyleIoUtil import PdbxStyleIoUtil
from mmcif_utils.style.PdbxMessageCategoryStyle import PdbxMessageCategoryStyle


class PdbxMessageIo(PdbxStyleIoUtil):
    ''' Methods for reading chemical component dictionary definitions including style details.

    '''

    def __init__(self, verbose=True, log=sys.stderr):
        super(PdbxMessageIo, self).__init__(styleObject=PdbxMessageCategoryStyle(), verbose=verbose, log=log)

        self.__verbose = verbose
        self.__debug = False
        self.__lfh = log
        self.__filePath = None

    def getCategory(self, catName='pdbx_deposition_message_info'):
        return self.getItemDictList(catName)

    def getMessageInfo(self):
        return self.getAttribDictList(catName='pdbx_deposition_message_info')

    def getFileReferenceInfo(self):
        return self.getAttribDictList(catName='pdbx_deposition_message_file_reference')

    def getOrigCommReferenceInfo(self):
        return self.getAttribDictList(catName='pdbx_deposition_message_origcomm_reference')

    def getMsgStatusInfo(self):
        return self.getAttribDictList(catName='pdbx_deposition_message_status')

    def read(self, filePath, logtag=""):
        """ Specify the file path for the target message data.
        """
        self.__filePath = filePath
        if self.readFile(self.__filePath, logtag=logtag):
            return self.setContainer(containerIndex=0)
        else:
            return False

    def complyStyle(self):
        return self.testStyleComplete(self.__lfh)

    def setBlock(self, blockId):
        return self.setContainer(containerName=blockId)

    def newBlock(self, blockId):
        self.newContainer(containerName=blockId, overWrite=False)
        self.setContainer(containerName=blockId)
        #
        self.newCategory('pdbx_deposition_message_info', container=self.getCurrentContainer())
        self.newCategory('pdbx_deposition_message_file_reference', container=self.getCurrentContainer())
        self.newCategory('pdbx_deposition_message_origcomm_reference', container=self.getCurrentContainer())
        self.newCategory('pdbx_deposition_message_status', container=self.getCurrentContainer())

    def update(self, catName, attributeName, value, iRow=0):
        return self.updateAttribute(catName, attributeName, value, iRow=iRow)

    def appendMessage(self, rowAttribDict):
        return self.appendRowByAttribute(rowAttribDict, catName='pdbx_deposition_message_info')

    def appendFileReference(self, rowAttribDict):
        return self.appendRowByAttribute(rowAttribDict, catName='pdbx_deposition_message_file_reference')

    def appendOrigCommReference(self, rowAttribDict):
        return self.appendRowByAttribute(rowAttribDict, catName='pdbx_deposition_message_origcomm_reference')

    def appendMsgReadStatus(self, rowAttribDict):
        return self.appendRowByAttribute(rowAttribDict, catName='pdbx_deposition_message_status')

    def write(self, filePath):
        return self.writeFile(filePath)

    def nextMessageOrdinal(self):
        nRows = int(self.getRowCount(catName='pdbx_deposition_message_info'))
        return nRows + 1

    def nextFileReferenceOrdinal(self):
        nRows = int(self.getRowCount(catName='pdbx_deposition_message_file_reference'))
        return nRows + 1

    def nextOrigCommReferenceOrdinal(self):
        nRows = int(self.getRowCount(catName='pdbx_deposition_message_origcomm_reference'))
        return nRows + 1



##
# File:  make-style-from-item-list.py
# Date:  Mar 5, 2018 jdw
#
# Create a template style definition from a list of mmCIF data items-
#
from __future__ import absolute_import
import sys
from mmcif.api.PdbxContainers import CifName

itemFileName = 'ITEMS'
with open(itemFileName, 'r') as ifh:
    #
    itList = []
    for line in ifh:
        itList.append(str(line[:-1]))

cList = []
iD = {}
for it in itList:
    catName = CifName.categoryPart(it)
    #
    if catName not in iD:
        cList.append(catName)
        iD[catName] = []
    iD[catName].append(it)

#
r = '''
##
# File: PdbxPLACEHOLDERCategoryStyle.py
# Date: 00-Mon-2015  J. Westbrook
#
# Updates:
#
##
"""
Abbreviated PDBx style defining data categories containing ....  details.

"""
__docformat__ = "restructuredtext en"
__author__ = "John Westbrook"
__email__ = "john.westbrook@rcsb.org"
__license__ = "Apache 2.0"
__version__ = "V0.01"


from mmcif_utils.style.PdbxCategoryStyleBase import PdbxCategoryStyleBase

class PdbxPLACEHOLDERCategoryStyle(PdbxCategoryStyleBase):
'''

sys.stdout.write("%s\n" % r)
sys.stdout.write("_styleId = 'PDBX_STYLE_PLACEHOLDER_V1'\n")
sys.stdout.write("_categoryInfo = [\n")
for catName in cList:
    sys.stdout.write("    ('%s', 'table'),\n" % catName)
sys.stdout.write("    ]\n")


sys.stdout.write("_cDict = {\n")
for catName in cList:
    sys.stdout.write("'%s': [\n" % catName)
    for it in iD[catName]:
        sys.stdout.write("    ('%s', '%%s', 'str', ''),\n" % (it))
    sys.stdout.write("     ],\n")
sys.stdout.write("    }\n")
#
sys.stdout.write("  _excludeList = []\n")
sys.stdout.write("  _suppressList = []\n")
r = '''
    #

    def __init__(self):
        super(PdbxPLACEHOLDERCategoryStyle, self).__init__(styleId=PdbxPLACEHOLDERCategoryStyle._styleId,
                                                         catFormatL=PdbxPLACEHOLDERCategoryStyle._categoryInfo,
                                                         catItemD=PdbxPLACEHOLDERCategoryStyle._cDict,
                                                         excludeList=PdbxPLACEHOLDERCategoryStyle._excludeList,
                                                         suppressList=PdbxPLACEHOLDERCategoryStyle._suppressList)
'''

sys.stdout.write("%s\n" % r)
